# Example file of using pytorch-lightning together with a NN

# Some import statements
import pytorch_lightning as pl
from pytorch_lightning.loggers import TensorBoardLogger

from utils import utilities
from utils import lightning_models
from utils import settings
from datetime import date


# Load data
train_loader, val_loader, test_loader = utilities.get_DataLoader()

# Torch model
for sample, label in train_loader:
    break
model_SDKN = lightning_models.SDKN(centers=sample[:5, :])

# Training
logger = TensorBoardLogger('tb_logs_' + date.today().strftime('%d.%m.%y') + '/',
                           name='model_SDKN')
trainer = pl.Trainer(max_epochs=settings.num_epochs_sdkn,
                     progress_bar_refresh_rate=settings.progress_bar_refresh_rate,
                     logger=logger,
                     log_every_n_steps=settings.log_every_n_steps,
                     gpus=1 if settings.ianscluster else 0
                     )

trainer.fit(model_SDKN, train_dataloader=train_loader, val_dataloaders=val_loader)

# Test
trainer.test(model_SDKN, dataloaders=test_loader)